--[[

Version 0.01 - Initial release of Concept code to wowace svn

Author : Levva of EU Khadgar 
	
--]]

local L = LibStub("AceLocale-3.0"):GetLocale("RaidInviter")
local AceAddon = LibStub("AceAddon-3.0")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
RaidInviter = AceAddon:NewAddon("RaidInviter", "AceConsole-3.0", "AceEvent-3.0")
local REVISION = tonumber(("$Revision: 6 $"):match("%d+"))

RaidInviter.textures = {}
RaidInviter.borders = {}
RaidInviter.fonts = {}
RaidInviter.sounds = {}
RaidInviter.frameBackdrop ={
		bgFile = "Interface/DialogFrame/UI-DialogBox-Background",
		edgeFile= "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32,
		insets = { left = 11, right = 12, top = 12, bottom = 11 }
	}

-------------------
-- Config details
-------------------

RaidInviter.defaults = {
	char = {
		message = "Welcome Home!",
		relativeTo = "UIParent",
		relativePoint = "CENTER",
		point = "CENTER",
		texture = "Blizzard",
		msgfont = "Friz Quadrata TT",
		msgfontsize = 24,
		msgfonteffect = "none",
		border = "None",
		barborder = "Blizzard Tooltip",
		xOffset = 0,
		yOffset = 0,
		scale = 1,
		debug = false,
	}
}

-----------------------------------------
-- Initialisation & Startup Routines
-----------------------------------------

function RaidInviter:OnInitialize()
	local AceConfigReg = LibStub("AceConfigRegistry-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")

	self.db = LibStub("AceDB-3.0"):New("RaidInviterDBPC", RaidInviter.defaults, "char")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RaidInviter", self:GetOptions(), {"RaidInviter", "ri"} )
	media.RegisterCallback(self, "LibSharedMedia_Registered")
	
	-- Add the options to blizzard frame (add them backwards so they show up in the proper order
	self.optionsFrame = AceConfigDialog:AddToBlizOptions("RaidInviter","RaidInviter")
	self.optionsFrame[L["About"]] = LibStub("LibAboutPanel").new("RaidInviter", "RaidInviter")
	self.db:RegisterDefaults(self.defaults)
	local version = GetAddOnMetadata("RaidInviter","Version")
	self.version = ("RaidInviter v%s (r%s)"):format(version, REVISION)
	self:Print(self.version.." Loaded.")
end

function RaidInviter:OnDisable()
    -- Called when the addon is disabled
end

function RaidInviter:OnEnable()
	self:LibSharedMedia_Registered()
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("UPDATE_BINDINGS")
	self:RegisterEvent("GUILD_ROSTER_UPDATE")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
end

function RaidInviter:LibSharedMedia_Registered()
	for k, v in pairs(media:List("statusbar")) do
		self.textures[v] = v
	end
	for k, v in pairs(media:List("border")) do
		self.borders[v] = v
	end
	for k, v in pairs(media:List("font")) do
		self.fonts[v] = v
	end
	for k, v in pairs(media:List("sound")) do
		self.sounds[v] = v
	end
end

----------------------
-- Event Routines
----------------------

function RaidInviter:PLAYER_ENTERING_WORLD()
	self:CreateInviteFrame()
end

function RaidInviter:UPDATE_BINDINGS()
--~ 	self:UpdateBindings()
end

function RaidInviter:GUILD_ROSTER_UPDATE()
	self.guildRoster = {}
	local guildMembers = GetNumGuildMembers(true)
	for index = 1, guildMembers do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
		self.guildRoster[name] = {}
		self.guildRoster[name].name = name
		self.guildRoster[name].rank = rank
		self.guildRoster[name].level = level
		self.guildRoster[name].zone = zone
		self.guildRoster[name].note = note
		self.guildRoster[name].officernote = officernote
		self.guildRoster[name].online = online
		self.guildRoster[name].status = status  -- "<AFK>", "<DND>", or ""
	end
	if self.InviteFrame:IsVisible() then
		self:UpdateInviteFrameGuildInfo()
	end
end

function RaidInviter:PARTY_MEMBERS_CHANGED()
	-- this fires when someone joins or leaves the raid or players are moved around in the raid
	-- when it fires we need to update the invite frame with their new status
end

----------------------
-- Frame Routines
----------------------

function RaidInviter:CreateInviteFrame()
	self.InviteFrame = CreateFrame("Frame", "RaidInviterBase", UIParent)
	self.InviteFrame:SetScale(self.db.char.scale)
	self.InviteFrame:SetFrameStrata("BACKGROUND")
	self.InviteFrame:SetWidth(700)
	self.InviteFrame:SetHeight(500)
	self.InviteFrame:SetBackdrop(self.frameBackdrop)
	self.InviteFrame:SetBackdropColor(1, 1, 1, 0)
	self.InviteFrame:SetMovable(true)
	self.InviteFrame:RegisterForDrag("LeftButton")
	self.InviteFrame:SetPoint(self.db.char.point, self.db.char.relativeTo, self.db.char.relativePoint, self.db.char.xOffset, self.db.char.yOffset)
	self.InviteFrame:SetScript("OnDragStart", 
		function()
			self.InviteFrame:StartMoving();
		end );
	self.InviteFrame:SetScript("OnDragStop",
		function()
			self.InviteFrame:StopMovingOrSizing();
			self.InviteFrame:SetScript("OnUpdate", nil);
			self:FinishedMoving(self.db.char, self.InviteFrame);
		end );
	self.InviteFrame:SetBackdropColor(0,0,0,1)
	self.InviteFrame.texture = self.InviteFrame:CreateTexture("RaidInviterFrameHeader", "ARTWORK")
	self.InviteFrame.texture:SetTexture("Interface/DialogFrame/UI-DialogBox-Header")
	self.InviteFrame.texture:SetHeight(64)
	self.InviteFrame.texture:SetWidth(350)
	self.InviteFrame.texture:SetPoint("TOP", 0, 12)
	
	self.InviteFrame.headertext = self.InviteFrame:CreateFontString("RaidInviterTitle", "ARTWORK", "GameFontNormal")
	self.InviteFrame.headertext:SetPoint("TOP", self.InviteFrame.texture, "TOP", 0, -14)
	self.InviteFrame.headertext:SetText("Raid Inviter")

	self.InviteFrame.event = self:CreateEventInfoFrame()
	self.InviteFrame.event.dropdown = self:CreateEventListDropdown()
	self.InviteFrame.playerList = self:CreatePlayerListFrame()
	self.InviteFrame.raidRoleComposition,  self.InviteFrame.raidClassComposition = self:CreateRaidCompositionFrames()
	self.InviteFrame:Show()
end

function RaidInviter:CreateEventInfoFrame()
	local eventInfo = CreateFrame("Frame", "RaidInviterEventInfo", self.InviteFrame)
	eventInfo:ClearAllPoints()
	eventInfo:SetWidth(350)
	eventInfo:SetHeight(100)
	eventInfo:SetPoint("CENTER", self.InviteFrame, "TOP", 0, -50)
	eventInfo:Show()
	return eventInfo
end

function RaidInviter:CreateEventListDropdown()
	local eventList = CreateFrame("Frame", "RaidInviterEventList", self.InviteFrame.event, "UIDropDownMenuTemplate")
	eventList:SetPoint("CENTER", self.InviteFrame.event)
	UIDropDownMenu_SetWidth(eventList, 300)
	UIDropDownMenu_Initialize(eventList, function(self) RaidInviter:PopulateEventList() end)
	eventList.text = self.InviteFrame.event:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
	eventList.text:SetFont("Friz Quadrata TT", 14)
	eventList.text:SetTextColor(1,1,1,1)
	eventList.text:SetPoint("RIGHT", eventList, "LEFT", -10, 0)
	eventList.text:SetText(L["Select Event"])
	eventList:Show()
	return eventList
end

function RaidInviter:PopulateEventList()
	local info = UIDropDownMenu_CreateInfo()
	local eventsToday = CalendarGetNumDayEvents(0, 0)
	if eventsToday == 0 then
		info.text = "No scheduled events today"
		info.value = -1 
		info.func = nil
		info.owner = this:GetParent()
		info.checked = nil
		info.icon = nil
		UIDropDownMenu_AddButton(info, 1)
	else
		for index = 1, eventsToday do
			CalendarOpenEvent(0, 0, index)
			local title, _, _, type = CalendarGetEventInfo()
			if type == CALENDAR_EVENTTYPE_RAID or type == CALENDAR_EVENTTYPE_DUNGEON then
				info.text = title
				info.value = index
				info.func = function() RaidInviter:EventListItem_OnClick() end
				info.owner = this:GetParent()
				info.checked = index == 1
				info.icon = nil
				UIDropDownMenu_AddButton(info, 1)
			end
		end
	end
end

function RaidInviter:EventListItem_OnClick()
	--- this function will read the list of invitees and populate the invite frame
	UIDropDownMenu_SetSelectedValue(this.owner, this.value)
	local guildRoster = self:LoadGuildRoster()
	CalendarOpenEvent(0, 0, this.value)
	local title, description, creator, type, instanceID, ownerFlag, _, weekday, month, day, year, hour, minute, _, _, _, _, _, _, lockedFlag, _, unreadFlag, myStatus = CalendarGetEventInfo()
	-- TODO setup event info frame with descriptions etc
	local invites = CalendarEventGetNumInvites()
	self.raidInvites = {}
	for index = 1, invites do
		local name, level, className, classTexture, signupStatus, modStatus = CalendarEventGetInvite(index)
--~ CALENDAR_INVITESTATUS_INVITED   = 1
--~ CALENDAR_INVITESTATUS_ACCEPTED  = 2
--~ CALENDAR_INVITESTATUS_DECLINED  = 3
--~ CALENDAR_INVITESTATUS_CONFIRMED = 4
--~ CALENDAR_INVITESTATUS_OUT       = 5
--~ CALENDAR_INVITESTATUS_STANDBY   = 6
--~ 	modStatus = "MODERATOR", "CREATOR", "" 
		self.raidInvites[name] = {}
		self.raidInvites[name].name = name
		self.raidInvites[name].level = level
		self.raidInvites[name].className = className
		self.raidInvites[name].classTexture = classTexture
		self.raidInvites[name].signupStatus = signupStatus
		self.raidInvites[name].modStatus = modStatus
		self.raidInvites[name].inviteStatus = 0 -- need a table with description, colour 0 = none, white, 1 = invited, yellow, 2 = declined, orange, 3 = offline, red, 4 = inraid, green
		self:UpdateInviteFrameGuildInfo()
	end
end

function RaidInviter:LoadGuildRoster()
	self.guildRoster = {}
	if IsInGuild() then
		GuildRoster()
	end
end

function RaidInviter:CreatePlayerListFrame()
	local playerList = CreateFrame("Frame", "RaidInviterPlayerList", self.InviteFrame)
	return playerList
end

function RaidInviter:CreateRaidCompositionFrames()
	local roleCompostion = CreateFrame("Frame", "RaidInviterRoleComposition", self.InviteFrame)
	local classCompostion = CreateFrame("Frame", "RaidInviterClassComposition", self.InviteFrame)
	return roleComposition, classComposition
end


function RaidInviter:UpdateInviteFrame()

end

function RaidInviter:UpdateInviteFrameGuildInfo()
	-- this function will update the raidInvites zone, online & afk/dnd status, 
	local invites = CalendarEventGetNumInvites()
	for index = 1, invites do
		local name = CalendarEventGetInvite(index)
		if self.guildRoster[name] then
			if self.raidInvites[name] then
				self.raidInvites[name].guildRank = self.guildRoster[name].rank
				self.raidInvites[name].note = self.guildRoster[name].note
				self.raidInvites[name].officernote = self.guildRoster[name].officernote
				self.raidInvites[name].zone = self.guildRoster[name].zone
				self.raidInvites[name].online = self.guildRoster[name].online
				self.raidInvites[name].active = self.guildRoster[name].active
			else
				self:Print("called updateInviteFrameGuildInfo and didn't find "..name)
			end
		end
	end
end
